package com.chelvc.framework.base.context;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Objects;

import com.chelvc.framework.common.util.JacksonUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

/**
 * Jackson上下文工具类
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
@Component
public class JacksonContextHolder implements ApplicationContextAware {
    /**
     * 全局对象序列化处理器实例
     */
    private static ObjectMapper GLOBAL_SERIALIZER;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        GLOBAL_SERIALIZER = applicationContext.getBean(ObjectMapper.class);
    }

    /**
     * 获取全局序列化处理器实例
     *
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper getGlobalSerializer() {
        return Objects.requireNonNull(GLOBAL_SERIALIZER, "Global serializer has not been initialized");
    }

    /**
     * 将对象写入输出流
     *
     * @param output 输出流对象
     * @param value  对象实例
     * @throws IOException I/O异常
     */
    public static void write(@NonNull OutputStream output, Object value) throws IOException {
        JacksonUtils.write(getGlobalSerializer(), output, value);
    }

    /**
     * 对象序列化
     *
     * @param object 目标对象实例
     * @return JSON字符串
     */
    public static String serialize(Object object) {
        return JacksonUtils.serialize(getGlobalSerializer(), object);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @return 对象实例
     */
    public static Object deserialize(String json) {
        return JacksonUtils.deserialize(getGlobalSerializer(), json, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @return 对象实例
     */
    public static Object deserialize(byte[] bytes) {
        return JacksonUtils.deserialize(getGlobalSerializer(), bytes, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(String json, Class<T> type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型引用对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(String json, TypeReference<T> type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型对象
     * @param <T>   对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(byte[] bytes, Class<T> type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型引用对象
     * @param <T>   对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(byte[] bytes, TypeReference<T> type) {
        return JacksonUtils.deserialize(getGlobalSerializer(), bytes, type);
    }
}
