package com.chelvc.framework.base.util;

import java.util.Objects;

import com.chelvc.framework.base.exception.ResourceUnavailableException;
import com.chelvc.framework.base.exception.ResourceUndefinedException;

/**
 * 资源工具类
 *
 * @author Woody
 * @date 2023/9/2
 */
public final class ResourceUtils {
    private ResourceUtils() {
    }

    /**
     * 资源非空校验
     *
     * @param resource 资源对象
     * @param <T>      对象类型
     * @return 非空资源
     */
    public static <T> T required(T resource) {
        return required(resource, "Resource undefined");
    }

    /**
     * 资源非空校验
     *
     * @param resource 资源对象
     * @param message  异常信息
     * @param <T>      对象类型
     * @return 非空资源
     */
    public static <T> T required(T resource, String message) {
        required(Objects.nonNull(resource), message);
        return resource;
    }

    /**
     * 资源非空校验
     *
     * @param resource 资源对象
     * @param message  异常信息
     * @param business 业务编码
     * @param <T>      对象类型
     * @return 非空资源
     */
    public static <T> T required(T resource, String message, String business) {
        required(Objects.nonNull(resource), message, business);
        return resource;
    }

    /**
     * 资源非空校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     */
    public static void required(boolean expression, String message) {
        required(expression, message, null);
    }

    /**
     * 资源非空校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     * @param business   业务编码
     */
    public static void required(boolean expression, String message, String business) {
        if (!expression) {
            throw new ResourceUndefinedException(message, business);
        }
    }

    /**
     * 资源可用性校验
     *
     * @param expression 验证表达式
     */
    public static void available(boolean expression) {
        available(expression, "Resource unavailable");
    }

    /**
     * 资源可用性校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     */
    public static void available(boolean expression, String message) {
        available(expression, message, null);
    }

    /**
     * 资源可用性校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     * @param business   业务编码
     */
    public static void available(boolean expression, String message, String business) {
        if (!expression) {
            throw new ResourceUnavailableException(message, business);
        }
    }
}
