package com.chelvc.framework.nacos.util;

import java.util.Collections;
import java.util.Map;
import java.util.Objects;
import java.util.StringTokenizer;
import java.util.stream.Collectors;

import com.alibaba.nacos.api.config.ConfigService;
import com.alibaba.nacos.api.exception.NacosException;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

/**
 * Nacos工具类
 *
 * @author Woody
 * @date 2023/4/5
 */
@Component
public final class NacosUtils implements ApplicationContextAware {
    /**
     * Nacos配置ID
     */
    private static String ID;

    /**
     * Nacos配置组
     */
    private static String GROUP;

    /**
     * 配置中心业务处理对象实例
     */
    private static ConfigService CONFIG_SERVICE;

    private NacosUtils() {
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        CONFIG_SERVICE = applicationContext.getBean(ConfigService.class);
        Environment environment = applicationContext.getEnvironment();
        if (StringUtils.isEmpty(ID = environment.getProperty("nacos.config.id"))) {
            ID = environment.getProperty("spring.application.name");
        }
        GROUP = environment.getProperty("spring.cloud.nacos.config.group", "DEFAULT_GROUP");
    }

    /**
     * 获取配置ID
     *
     * @return 配置ID
     */
    public static String getId() {
        return Objects.requireNonNull(ID, "Nacos config id has not been initialized");
    }

    /**
     * 获取配置分组
     *
     * @return 配置分组
     */
    public static String getGroup() {
        return Objects.requireNonNull(GROUP, "Nacos config group has not been initialized");
    }

    /**
     * 获取配置中心业务处理对象实例
     *
     * @return 配置中心业务处理对象实例
     */
    public static ConfigService getConfigService() {
        return Objects.requireNonNull(CONFIG_SERVICE, "Nacos config service has not been initialized");
    }

    /**
     * 获取配置
     *
     * @return 配置键/值映射表
     */
    public static Map<String, String> getConfiguration() {
        return getConfiguration(getId());
    }

    /**
     * 获取配置
     *
     * @param id 配置ID
     * @return 配置键/值映射表
     */
    public static Map<String, String> getConfiguration(@NonNull String id) {
        return getConfiguration(id, getGroup());
    }

    /**
     * 获取配置
     *
     * @param id    配置ID
     * @param group 配置分组
     * @return 配置键/值映射表
     */
    public static Map<String, String> getConfiguration(@NonNull String id, @NonNull String group) {
        // 获取配置信息
        String content;
        ConfigService service = getConfigService();
        try {
            content = service.getConfig(id, group, 3000);
        } catch (NacosException e) {
            throw new RuntimeException(e);
        }
        if (StringUtils.isEmpty(content)) {
            return Collections.emptyMap();
        }

        // 解析配置信息映射
        Map<String, String> configuration = Maps.newHashMap();
        StringTokenizer tokenizer = new StringTokenizer(content, StringUtils.ENTER);
        while (tokenizer.hasMoreElements()) {
            String[] splits = tokenizer.nextToken().split(StringUtils.EQUAL);
            String key = splits[0], value = splits.length > 1 ? splits[1] : null;
            configuration.put(key, value);
        }
        return configuration;
    }

    /**
     * 初始化配置信息
     *
     * @param key   配置键
     * @param value 配置值
     */
    public static void initializeConfiguration(@NonNull String key, @NonNull String value) {
        initializeConfiguration(getId(), key, value);
    }

    /**
     * 初始化配置信息
     *
     * @param id    配置ID
     * @param key   配置键
     * @param value 配置值
     */
    public static void initializeConfiguration(@NonNull String id, @NonNull String key, @NonNull String value) {
        initializeConfiguration(id, getGroup(), key, value);
    }

    /**
     * 初始化配置信息
     *
     * @param id    配置ID
     * @param group 配置分组
     * @param key   配置键
     * @param value 配置值
     */
    public static void initializeConfiguration(@NonNull String id, @NonNull String group, @NonNull String key,
                                               @NonNull String value) {
        initializeConfiguration(id, group, ImmutableMap.of(key, value));
    }

    /**
     * 初始化配置信息
     *
     * @param configuration 配置信息键/值映射表
     */
    public static void initializeConfiguration(@NonNull Map<String, String> configuration) {
        initializeConfiguration(getId(), configuration);
    }

    /**
     * 初始化配置信息
     *
     * @param id            配置ID
     * @param configuration 配置信息键/值映射表
     */
    public static void initializeConfiguration(@NonNull String id, @NonNull Map<String, String> configuration) {
        initializeConfiguration(id, getGroup(), configuration);
    }

    /**
     * 初始化配置信息
     *
     * @param id            配置ID
     * @param group         配置分组
     * @param configuration 配置信息键/值映射表
     */
    public static void initializeConfiguration(@NonNull String id, @NonNull String group,
                                               @NonNull Map<String, String> configuration) {
        ConfigService service = getConfigService();
        String content = configuration.entrySet().stream().map(entry ->
                entry.getKey() + StringUtils.EQUAL + ObjectUtils.ifNull(entry.getValue(), StringUtils.EMPTY)
        ).collect(Collectors.joining(StringUtils.ENTER));
        try {
            service.publishConfig(id, group, content);
        } catch (NacosException e) {
            throw new RuntimeException(e);
        }
    }
}
