package com.chelvc.framework.oauth.context;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;

import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Sets;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.security.oauth2.common.OAuth2RefreshToken;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.jwt.JwtDecoder;
import org.springframework.security.oauth2.provider.ClientDetails;
import org.springframework.stereotype.Component;

/**
 * OAuth上下文工具类
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
@Component
public class OauthContextHolder implements ServletRequestListener, ApplicationContextAware {
    /**
     * JWT身份标识
     */
    public static final String JWT_CLAIM_JTI = "jti";

    /**
     * JWT手机号标识
     */
    public static final String JWT_CLAIM_MOBILE = "mobile";

    /**
     * 业务类型标识
     */
    public static final String JWT_CLAIM_BUSINESS = "business";

    /**
     * 用户账号标识
     */
    public static final String JWT_CLAIM_USER_NAME = "user_name";

    /**
     * 用户权限标识
     */
    public static final String JWT_CLAIM_AUTHORITIES = "authorities";

    /**
     * JWT解码器
     */
    private static JwtDecoder JWT_DECODER;

    /**
     * 认证主体上下文
     */
    private static final ThreadLocal<Object> PRINCIPAL_CONTEXT = new ThreadLocal<>();

    /**
     * 客户端详情上下文
     */
    private static final ThreadLocal<ClientDetails> CLIENT_DETAILS_CONTEXT = new ThreadLocal<>();

    /**
     * 刷新令牌上下文
     */
    private static final ThreadLocal<OAuth2RefreshToken> REFRESH_TOKEN_CONTEXT = new ThreadLocal<>();

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        JWT_DECODER = applicationContext.getBean(JwtDecoder.class);
    }

    /**
     * 获取JWT解码器
     *
     * @return JWT解码器
     */
    public static JwtDecoder getJwtDecoder() {
        return Objects.requireNonNull(JWT_DECODER, "Jwt decoder has not been initialized");
    }

    /**
     * 获取认证主体
     *
     * @return 认证主体
     */
    public static Object getPrincipal() {
        return PRINCIPAL_CONTEXT.get();
    }

    /**
     * 设置认证主体
     *
     * @param principal 认证主体
     */
    public static void setPrincipal(Object principal) {
        PRINCIPAL_CONTEXT.set(principal);
    }

    /**
     * 获取客户端详情
     *
     * @return 客户端详情
     */
    public static ClientDetails getClientDetails() {
        return CLIENT_DETAILS_CONTEXT.get();
    }

    /**
     * 设置客户端详情
     *
     * @param details 客户端详情
     */
    public static void setClientDetails(ClientDetails details) {
        CLIENT_DETAILS_CONTEXT.set(details);
    }

    /**
     * 获取当前刷新令牌
     *
     * @return 刷新令牌实例
     */
    public static OAuth2RefreshToken getRefreshToken() {
        return REFRESH_TOKEN_CONTEXT.get();
    }

    /**
     * 设置当前刷新令牌
     *
     * @param refreshToken 刷新令牌实例
     */
    public static void setRefreshToken(OAuth2RefreshToken refreshToken) {
        REFRESH_TOKEN_CONTEXT.set(refreshToken);
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_USER_NAME), Long::parseLong);
    }

    /**
     * 获取JWT身份标识
     *
     * @param jwt JWT对象
     * @return JWT身份标识
     */
    public static String getJti(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_JTI), (String) null);
    }

    /**
     * 获取手机号
     *
     * @param jwt JWT对象
     * @return 手机号
     */
    public static String getMobile(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_MOBILE), (String) null);
    }

    /**
     * 获取业务类型
     *
     * @param jwt JWT对象
     * @return 业务类型
     */
    public static String getBusiness(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_BUSINESS), (String) null);
    }

    /**
     * 获取业务类型
     *
     * @param token 刷新令牌
     * @return 业务类型
     */
    public static String getBusiness(OAuth2RefreshToken token) {
        String value = ObjectUtils.ifNull(token, OAuth2RefreshToken::getValue);
        return getBusiness(StringUtils.ifEmpty(value, getJwtDecoder()::decode));
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(Jwt jwt) {
        if (jwt == null) {
            return Collections.emptySet();
        }
        List<String> authorities = jwt.getClaimAsStringList(JWT_CLAIM_AUTHORITIES);
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }

    /**
     * 构建访问令牌标识
     *
     * @param principal 主体标识
     * @param terminal  终端信息
     * @return 令牌标识
     */
    public static String buildAccessTokenKey(@NonNull Object principal, @NonNull Terminal terminal) {
        return String.format("token:%s:%s", terminal, principal);
    }

    /**
     * 清空OAuth上下文
     */
    public static void clearOauthContext() {
        PRINCIPAL_CONTEXT.remove();
        CLIENT_DETAILS_CONTEXT.remove();
        REFRESH_TOKEN_CONTEXT.remove();
    }

    @Override
    public void requestDestroyed(ServletRequestEvent event) {
        clearOauthContext();
    }
}
