package com.chelvc.framework.rocketmq.model;

import java.time.Duration;
import java.util.Date;

import lombok.NonNull;

/**
 * 消息延时类型枚举
 *
 * @author Woody
 * @date 2023/4/5
 */
public enum Delay {
    /**
     * 1秒
     */
    S1(1, 1),

    /**
     * 5秒
     */
    S5(2, 5),

    /**
     * 10秒
     */
    S10(3, 10),

    /**
     * 30秒
     */
    S30(4, 30),

    /**
     * 1分钟
     */
    M1(5, 60),

    /**
     * 2分钟
     */
    M2(6, 120),

    /**
     * 3分钟
     */
    M3(7, 180),

    /**
     * 4分钟
     */
    M4(8, 240),

    /**
     * 5分钟
     */
    M5(9, 300),

    /**
     * 6分钟
     */
    M6(10, 360),

    /**
     * 7分钟
     */
    M7(11, 420),

    /**
     * 8分钟
     */
    M8(12, 480),

    /**
     * 9分钟
     */
    M9(13, 540),

    /**
     * 10分钟
     */
    M10(14, 600),

    /**
     * 20分钟
     */
    M20(15, 1200),

    /**
     * 30分钟
     */
    M30(16, 1800),

    /**
     * 1小时
     */
    H1(17, 3600),

    /**
     * 2小时
     */
    H2(18, 7200);

    /**
     * 延时类型值数组
     */
    private static final Delay[] VALUES = Delay.values();

    /**
     * 延时等级
     */
    private final int level;

    /**
     * 延时时间（秒）
     */
    private final long time;

    Delay(int level, long time) {
        this.level = level;
        this.time = time;
    }

    /**
     * 根据延时等级获取延时类型
     *
     * @param level 延时等级
     * @return 延时类型
     */
    public static Delay of(int level) {
        return VALUES[level - 1];
    }

    /**
     * 获取时间接近的延时类型
     *
     * @param time 延时时间（秒）
     * @return 延时类型
     */
    public static Delay close(long time) {
        if (time <= 1) {
            return Delay.S1;
        }
        Delay delay = null;
        int median = VALUES.length / 2;
        int i = time >= VALUES[median].time ? median : 0;
        for (; i < VALUES.length; i++) {
            Delay value = VALUES[i];
            if (time < value.time) {
                break;
            }
            delay = value;
        }
        return delay == null ? Delay.S1 : delay;
    }

    /**
     * 获取目标日期接近的延时类型
     *
     * @param date 目标日期
     * @return 延时类型
     */
    public static Delay close(@NonNull Date date) {
        return close((date.getTime() - System.currentTimeMillis()) / 1000);
    }

    /**
     * 获取时间周期接近的延时类型
     *
     * @param duration 时间周期
     * @return 延时类型
     */
    public static Delay close(@NonNull Duration duration) {
        return close(duration.getSeconds());
    }

    /**
     * 获取延时等级
     *
     * @return 延时等级
     */
    public int level() {
        return this.level;
    }

    /**
     * 获取延时时间（秒）
     *
     * @return 延时时间
     */
    public long time() {
        return this.time;
    }

    /**
     * 获取延时周期
     *
     * @return 时间周期
     */
    public Duration duration() {
        return Duration.ofSeconds(this.time);
    }
}
